@@ -1,5 +1,5 @@
 /* hdparm.c - Command line interface to get/set hard disk parameters */
-/*          - by Mark Lord (C) 1994-2002 -- freely distributable */
+/*          - by Mark Lord (C) 1994-2004 -- freely distributable */
 
 #include <unistd.h>
 #include <string.h>
@@ -26,7 +26,7 @@
 
 extern const char *minor_str[];
 
-#define VERSION "v5.4"
+#define VERSION "v5.5"
 
 #undef DO_FLUSHCACHE		/* under construction: force cache flush on -W0 */
 
@@ -49,7 +49,7 @@ extern const char *minor_str[];
 
 char *progname;
 static int verbose = 0, get_identity = 0, get_geom = 0, noisy = 1, quiet = 0;
-static int flagcount = 0, do_flush = 0, is_scsi_hd = 0, is_xt_hd = 0;
+static int flagcount = 0, do_flush = 0, is_scsi_hd = 0, is_scsi_cd = 0, is_xt_hd = 0;
 static int do_ctimings, do_timings = 0;
 
 static unsigned long set_readahead= 0, get_readahead= 0, readahead= 0;     
@@ -190,11 +190,7 @@ static void dump_identity (const struct hd_driveid *id)
 	}
 	if (!(id->field_valid&1))
 		printf(" (maybe):");
-#if __BYTE_ORDER == __BIG_ENDIAN
-	capacity = (id->cur_capacity0 << 16) | id->cur_capacity1;
-#else
 	capacity = (id->cur_capacity1 << 16) | id->cur_capacity0;
-#endif
 	printf(" CurCHS=%u/%u/%u, CurSects=%lu", id->cur_cyls, id->cur_heads, id->cur_sectors, capacity);
 	printf(", LBA=%s", YN(id->capability&2));
 	if (id->capability&2)
@@ -277,7 +273,7 @@ static void dump_identity (const struct hd_driveid *id)
 	if (id_regs[82]&0x20)
 		printf(" WriteCache=%s",(id_regs[85]&0x20) ? "enabled" : "disabled");
 #ifdef __NEW_HD_DRIVE_ID
-	if ((id->minor_rev_num && id->minor_rev_num <= 31) || (id->major_rev_num && id->minor_rev_num <= 31)) {
+	if (id->minor_rev_num || id->major_rev_num) {
 		printf("\n Drive conforms to: ");
 		if (id->minor_rev_num <= 31)
 			printf("%s: ", minor_str[id->minor_rev_num]);
@@ -502,7 +498,7 @@ quit:
 
 void no_scsi (void)
 {
-	if (is_scsi_hd) {
+	if (is_scsi_hd || is_scsi_cd) {
 		fputs (" operation not supported on SCSI disks\n", stderr);
 		exit(EINVAL);
 	}
@@ -715,6 +711,9 @@ void process_dev (char *devname)
 	}
 
 	switch(major(stat_buf.st_rdev)) {
+	case SCSI_CDROM_MAJOR:
+		is_scsi_cd = 1;
+		break;
 #ifdef SCSI_DISK0_MAJOR
 	case (SCSI_DISK0_MAJOR):
 	case (SCSI_DISK1_MAJOR):
