@@ -231,7 +231,7 @@ const char *ata1_cfg_str[] = {			/* word 0 in ATA-1 mode */
 /* NOVAL_0 or  NOVAL_1 means device does not report version */
 
 /* word 81: minor version number */
-#define MINOR_MAX		0x1C
+#define MINOR_MAX		0x1f
 const char *minor_str[] = {				/* word 81 value: */
 	"device does not report version",		/* 0x0000	*/
 	"ATA-1 X3T9.2 781D prior to revision 4",	/* 0x0001	*/
@@ -297,7 +297,9 @@ const char actual_ver[] = {
 	0,		/* 0x001a	WARNING:	*/
 	6,		/* 0x001b	WARNING:	*/
 	6,		/* 0x001c	WARNING:	*/
-	0		/* 0x001d-0xfffe    		*/
+	0,		/* 0x001d	WARNING:	*/
+	0,		/* 0x001e	WARNING:	*/
+	0		/* 0x001f-0xfffe    		*/
 };
 
 /* words 82-84: cmds/feats supported */
@@ -420,8 +422,8 @@ void identify (__u16 *id_supplied, const char *devname)
 	__u16 dev = NO_DEV, eqpt = NO_DEV;
 	__u8  have_mode = 0, err_dma = 0;
 	__u8  chksum = 0;
-	__u32 ll, mm, nn, oo;
-	__u64 bbbig; /* (:) */
+	__u32 ll, mm, nn;
+	__u64 bb, bbbig; /* (:) */
 
 	if (id_supplied) {
 #if __BYTE_ORDER == __BIG_ENDIAN
@@ -638,8 +640,9 @@ void identify (__u16 *id_supplied, const char *devname)
 		}
 	} else {
 	  	/* addressing...CHS? See section 6.2 of ATA specs 4 or 5 */
-		ll = (__u32)val[LBA_SECTS_MSB] << 16 | val[LBA_SECTS_LSB];
-		mm = 0; bbbig = 0;
+		ll = 0; mm = 0; bb = 0; bbbig = 0;
+		if (val[CAPAB_0] & LBA_SUP)
+			ll = (__u32)val[LBA_SECTS_MSB] << 16 | val[LBA_SECTS_LSB];
 	  	if ( (ll > 0x00FBFC10) && (!val[LCYLS])) {
 		  	printf("\tCHS addressing not supported\n");
 		} else {
@@ -648,6 +651,10 @@ void identify (__u16 *id_supplied, const char *devname)
 			printf("\tcylinders\t%u\t%u\n",val[LCYLS],jj?val[LCYLS_CUR]:0);
 			printf("\theads\t\t%u\t%u\n",val[LHEADS],jj?val[LHEADS_CUR]:0);
 			printf("\tsectors/track\t%u\t%u\n",val[LSECTS],jj?val[LSECTS_CUR]:0);
+			if(jj)
+				bb = (__u64)val[LCYLS_CUR] * val[LHEADS_CUR] * val[LSECTS_CUR];
+			else
+				bb = (__u64)val[LCYLS] * val[LHEADS] * val[LSECTS];
 			printf("\t--\n");
 			if((min_std == 1) && (val[TRACK_BYTES] || val[SECT_BYTES])) {
 				printf("\tbytes/track: %u",val[TRACK_BYTES]);
@@ -655,26 +662,30 @@ void identify (__u16 *id_supplied, const char *devname)
 			}
 			if(jj) {
 				mm = (__u32)val[CAPACITY_MSB] << 16 | val[CAPACITY_LSB];
-				if(like_std < 3) { /* check Endian of capacity bytes */
-					nn = val[LCYLS_CUR] * val[LHEADS_CUR] * val[LSECTS_CUR];
-					oo = (__u32)val[CAPACITY_LSB] << 16 | val[CAPACITY_MSB];
-					if(abs(mm - nn) > abs(oo - nn))
-						mm = oo;
+				/* ATA-1 is ambiguous on ordering of words 57 & 58 */
+				if(like_std < 3) {
+					nn = (__u32)val[CAPACITY_LSB] << 16 | val[CAPACITY_MSB];
+					/* check Endian of capacity bytes */
+					if(abs(mm - bb) > abs(nn - bb))
+						mm = nn;
 				}
 				printf("\tCHS current addressable sectors:%11u\n",mm);
 			} 
 		}
+		if (val[CAPAB_0] & LBA_SUP) {
 		/* LBA addressing */
-		printf("\tLBA    user addressable sectors:%11u\n",ll);
-		if( ((val[CMDS_SUPP_1] & VALID) == VALID_VAL) &&
-		     (val[CMDS_SUPP_1] & SUPPORT_48_BIT) ) {
-			bbbig = (__u64)val[LBA_64_MSB] << 48 | 
-			        (__u64)val[LBA_48_MSB] << 32 |
-			        (__u64)val[LBA_MID] << 16 | 
-				val[LBA_LSB] ;
-			printf("\tLBA48  user addressable sectors:%11llu\n",bbbig);
+			printf("\tLBA    user addressable sectors:%11u\n",ll);
+			if( ((val[CMDS_SUPP_1] & VALID) == VALID_VAL) &&
+			     (val[CMDS_SUPP_1] & SUPPORT_48_BIT) ) {
+				bbbig = (__u64)val[LBA_64_MSB] << 48 | 
+				        (__u64)val[LBA_48_MSB] << 32 |
+				        (__u64)val[LBA_MID] << 16 | 
+					val[LBA_LSB] ;
+				printf("\tLBA48  user addressable sectors:%11llu\n",bbbig);
+			}
 		}
 		if (!bbbig) bbbig = (__u64)(ll>mm ? ll : mm); /* # 512 byte blocks */
+		if (!bbbig) bbbig = bb;
 		printf("\tdevice size with M = 1024*1024: %11llu MBytes\n",bbbig>>11);
 		bbbig = (bbbig<<9)/1000000;
 		printf("\tdevice size with M = 1000*1000: %11llu MBytes ",bbbig);
@@ -699,7 +710,9 @@ void identify (__u16 *id_supplied, const char *devname)
 		if(val[CAPAB_0] & IORDY_OFF) printf("(can");
 		else			     printf("(cannot");
 		printf(" be disabled)");
-	} else printf("no IORDY"); 
+	} else {
+		printf("IORDY not likely"); 
+	}
 	printf("\n");
 	if((like_std == 1) && val[BUF_TYPE]) {
 		kk = val[BUF_TYPE];
@@ -783,21 +796,24 @@ void identify (__u16 *id_supplied, const char *devname)
 	if(!(val[CAPAB_0] & DMA_SUP)) {
 		printf("not supported\n");
 	} else {
-		if(val[DMA_MODE] && !val[SINGLE_DMA] && !val[MULTI_DMA]) 
-			printf(" sdma%u\n",(val[DMA_MODE] & MODE) >> 8);
-		if(val[SINGLE_DMA]) {
-			jj = val[SINGLE_DMA];   kk = val[SINGLE_DMA] >> 8;
-			err_dma += mode_loop(jj,kk,'s',&have_mode);
+		if(val[DMA_MODE] && !val[SINGLE_DMA] && !val[MULTI_DMA]) {
+			printf("sdma%u",(val[DMA_MODE] & MODE) >> 8);
+		} else {
+			if(val[SINGLE_DMA]) {
+				jj = val[SINGLE_DMA];   kk = val[SINGLE_DMA] >> 8;
+				err_dma += mode_loop(jj,kk,'s',&have_mode);
+			}
+			if(val[MULTI_DMA]) {
+				jj = val[MULTI_DMA];   kk = val[MULTI_DMA] >> 8;
+				err_dma += mode_loop(jj,kk,'m',&have_mode);
+			}
+			if((val[WHATS_VALID] & OK_W88) && val[ULTRA_DMA]) {
+				jj = val[ULTRA_DMA];   kk = val[ULTRA_DMA] >> 8;
+				err_dma += mode_loop(jj,kk,'u',&have_mode);
+			}
+			if(err_dma || !have_mode)
+				printf("(?)");
 		}
-		if(val[MULTI_DMA]) {
-			jj = val[MULTI_DMA];   kk = val[MULTI_DMA] >> 8;
-			err_dma += mode_loop(jj,kk,'m',&have_mode);
-		}
-		if((val[WHATS_VALID] & OK_W88) && val[ULTRA_DMA]) {
-			jj = val[ULTRA_DMA];   kk = val[ULTRA_DMA] >> 8;
-			err_dma += mode_loop(jj,kk,'u',&have_mode);
-		}
-		if(err_dma || !have_mode) printf("(?)");
 		printf("\n");
 
 		if((dev == ATAPI_DEV) && (eqpt != CDROM) && (val[CAPAB_0] & DMA_IL_SUP)) 
@@ -826,7 +842,7 @@ void identify (__u16 *id_supplied, const char *devname)
 			jj >>=1;
 		}
 		printf("\n");
-	} else if(((min_std < 5) || (eqpt == CDROM)) && (val[PIO_MODE] & MODE) ) {
+	} else if(((min_std < 5) || (eqpt == CDROM)) && ((val[PIO_MODE]>>8) <= 2)) {
 		for(ii = 0; ii <= val[PIO_MODE]>>8; ii++) {
 			printf("pio%d ",ii);
 		}
